==> glowroot-test-harness-0.8.3-sources.jar: org/glowroot/transaction/model/TimerNameImplBase.java <==
/*
 * Copyright 2014-2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glowroot.transaction.model;

import java.util.concurrent.atomic.AtomicInteger;

import javax.annotation.Nullable;

import org.glowroot.shaded.google.common.annotations.VisibleForTesting;
import org.immutables.value.Value;

import org.glowroot.api.TimerName;

@Value.Immutable
public abstract class TimerNameImplBase implements TimerName {

    private static final AtomicInteger nextSpecialHashCode = new AtomicInteger();

    @VisibleForTesting
    @Value.Parameter
    public abstract String name();

    @Value.Default
    public boolean extended() {
        return false;
    }

    @Value.Derived
    public @Nullable TimerNameImpl extendedTimer() {
        if (extended()) {
            return null;
        }
        return TimerNameImpl.builder()
                .name(name())
                .extended(true)
                .build();
    }

    @Value.Derived
    int specialHashCode() {
        return nextSpecialHashCode.getAndIncrement();
    }
}

==> glowroot-test-harness-0.8.4-sources.jar: org/glowroot/transaction/model/TimerNameImplBase.java <==
/*
 * Copyright 2014-2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glowroot.transaction.model;

import java.util.concurrent.atomic.AtomicInteger;

import javax.annotation.Nullable;

import org.glowroot.shaded.google.common.annotations.VisibleForTesting;
import org.immutables.value.Value;

import org.glowroot.plugin.api.transaction.TimerName;
import org.glowroot.transaction.model.TimerNameImpl;

@Value.Immutable
public abstract class TimerNameImplBase implements TimerName {

    private static final AtomicInteger nextSpecialHashCode = new AtomicInteger();

    @VisibleForTesting
    @Value.Parameter
    public abstract String name();

    @Value.Default
    public boolean extended() {
        return false;
    }

    @Value.Derived
    public @Nullable TimerNameImpl extendedTimer() {
        if (extended()) {
            return null;
        }
        return TimerNameImpl.builder()
                .name(name())
                .extended(true)
                .build();
    }

    @Value.Derived
    int specialHashCode() {
        return nextSpecialHashCode.getAndIncrement();
    }
}
